package com.persagy.job;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.cache.CreatedAlarmIdsCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.AlarmConditionState;
import com.persagy.entity.AlarmRecord;
import com.persagy.entity.NettyMessage;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.entity.v2.DmpMessage;
import com.persagy.enumeration.AlarmDefineStateEnum;
import com.persagy.enumeration.NettyMsgTypeEnum;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.utils.DateUtils;
import com.persagy.utils.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.util.Collections;
import java.util.Date;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicLong;

import static org.quartz.SimpleScheduleBuilder.simpleSchedule;

@DisallowConcurrentExecution
@PersistJobDataAfterExecution
@Slf4j
public class AlarmExpireJob extends QuartzJobBean {

    @Autowired
    private GroupNettyClient groupNettyClient;
    @Autowired
    AlarmRecordRepository alarmRecordRepository;
    @Autowired
    AlarmInfoCache alarmInfoCache;
    @Autowired
    CreatedAlarmIdsCache createdAlarmIdsCache;
    /**
     * 报警记录信息详情
     */
    private String alarmRecord;
    /**
     * 重试次数
     */
    private String refire;
    /**
     * 过期时间
     */
    private String expireTime;
    /**
     * 报警定义标识（itemcode+objId）
     */
    private String defineId;

    public String getState() {
        return state;
    }

    public void setState(String state) {
        this.state = state;
    }

    /**
     * 报警状态（2-恢复 3-过期）
     */
    private String state;

    public String getEndTime() {
        return endTime;
    }

    public void setEndTime(String endTime) {
        this.endTime = endTime;
    }

    public String getEndInfo() {
        return endInfo;
    }

    public void setEndInfo(String endInfo) {
        this.endInfo = endInfo;
    }

    /**
     * 恢复时间
     */
    private String endTime;
    /**
     * 恢复时刻信息
     */
    private String endInfo;
    /**
     * 集团编码
     */
    @Value("${group.code}")
    public String groupCode;

    private AtomicLong nums = new AtomicLong(1L);

    /**
     * @description: 执行恢复/过期定时任务
     * @param: context
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/22 上午9:38
     * @version: V1.0
     */
    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        try {
            String jobType = AlarmDefineStateEnum.NOT_DEAL.getValue().equals(state) ? "恢复" : "过期";
            JobDataMap mergedJobDataMap = context.getMergedJobDataMap();
            log.info("定时任务开始{}", alarmRecord);
            log.info("定时任务为{}任务", jobType);
            log.warn("refireCount:[{}],过期/恢复时间：[{}/{}],实际执行时间：[{}]",
                    context.getRefireCount(),
                    StringUtil.getString(expireTime),
                    StringUtil.getString(endTime),
                    DateUtil.format(context.getFireTime(), DatePattern.ISO8601_PATTERN));
            if (StringUtils.isNotBlank(alarmRecord)) {
                ZktAlarmRecordDO zktAlarmRecordDO = StringUtil.tranferItemToDTO(alarmRecord, ZktAlarmRecordDO.class);
                String alarmId = zktAlarmRecordDO.getAlarmId();

                // 立即过期的场景，过期的时候可能还没有报警记录ID,需要延迟重新执行
                if (!createdAlarmIdsCache.contains(alarmId)) {
                    log.info("已创建的报警id中不包含[{}], 定时任务将在5分钟后重新执行", alarmId);
                    log.info("refire:[{}]", refire);
                    mergedJobDataMap.put("refire", String.valueOf(StringUtil.getInt(refire) + 1));
                    reFireJob(context, mergedJobDataMap);
                    log.info("----------------结束---------------------");
                    return;
                }

                log.info("报警参数为：[{}]", zktAlarmRecordDO);

                AlarmRecord message = AlarmRecord.builder()
                        .id(alarmId)
                        .state(StringUtil.getInt(state))
                        .groupCode(groupCode)
                        .projectId(zktAlarmRecordDO.getProjectId())
                        .build();
                // 恢复任务发送消息时，带上恢复信息
                if (AlarmDefineStateEnum.NOT_DEAL.getValue().equals(state)) {
                    message.setEndInfo(endInfo);
                    message.setEndTime(DateUtils.parseDate(endTime));
                }

                DmpMessage dmpMessage = new DmpMessage();
                dmpMessage.setDefaultValue();
                dmpMessage.setMid(alarmId);
                dmpMessage.setProjectId(zktAlarmRecordDO.getProjectId());
                dmpMessage.setGroupCode(groupCode);
                dmpMessage.setExts(JSONObject.parseObject(JSONObject.toJSONString(message)));
                //{"id","123", "state":1, "groupCode":"wd", "projectId":"Pj123"}
                log.info("定时任务执行完成，向云端推送{}消息", jobType);
                // TODO: 2021/11/17 向mq推送消息
                // TODO: 2021/11/17 移除缓存信息，将报警定义状态更新到redis
                // 删除缓存中报警id
                createdAlarmIdsCache.remove(alarmId);

                String defineId = zktAlarmRecordDO.getDefinitionId();
                synchronized (defineId.intern()) {
                    // 判断报警是否仍是创建任务时的报警，如果不是了，任务结束
                    Optional<ZktAlarmRecordDO> currentRecordOptional = alarmRecordRepository.findById(defineId);
                    if (!currentRecordOptional.isPresent()) {
                        log.info("定时任务执行完成");
                        return;
                    }
                    ZktAlarmRecordDO alarmRecordDO = currentRecordOptional.get();
                    if (!alarmId.equals(alarmRecordDO.getAlarmId())) {
                        log.info("定时任务执行完成");
                        return;
                    }

                    // 过期任务执行时，如果报警仍是创建任务时的报警，重置报警定义的状态
                    if (AlarmDefineStateEnum.EXPIRED.getValue().equals(state)) {
                        AlarmConditionState alarmConditionState = new AlarmConditionState(this.defineId);
                        alarmConditionState.setAlarmStartTime("");
                        alarmInfoCache.setAlarmState(this.defineId, alarmConditionState);
                    }

                    // 如果报警仍是创建任务时的报警，数据库中删除这条报警定义
                    alarmRecordRepository.deleteById(defineId);
                }
            }

        } catch (Exception e) {
            log.error("定时任务执行失败", e);
        }

    }


    private void reFireJob(JobExecutionContext context, JobDataMap mergedJobDataMap) {
        try {
            if (StringUtil.getInt(refire) > 1) {
                log.error("重新执行依然失败,信息为：{}[{}]", state, alarmRecord);
            } else {
                Scheduler scheduler = context.getScheduler();
                Trigger trigger = context.getTrigger();
                DateTime dateTime = DateUtil.offsetSecond(new Date(), 300);
                Trigger newTrigger = TriggerBuilder.newTrigger()
                        .startAt(dateTime.toJdkDate())
                        .withIdentity(trigger.getKey())
                        .usingJobData(mergedJobDataMap)
                        .withSchedule(simpleSchedule().withMisfireHandlingInstructionIgnoreMisfires()).build();
                Date rescheduleJob = scheduler.rescheduleJob(trigger.getKey(), newTrigger);
            }
        } catch (Exception e) {
            log.error("重新执行定时任务失败！", e);
        }
    }

    public String getAlarmRecord() {
        return alarmRecord;
    }

    public void setAlarmRecord(String alarmRecord) {
        this.alarmRecord = alarmRecord;
    }

    public String getRefire() {
        return refire;
    }

    public void setRefire(String refire) {
        this.refire = refire;
    }

    public String getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(String expireTime) {
        this.expireTime = expireTime;
    }

    public String getDefineId() {
        return defineId;
    }

    public void setDefineId(String defineId) {
        this.defineId = defineId;
    }
}
